##A. CREDITS -------------------------------------------------------------------

#THIS FILE PROVIDES EXERCISES FOR THE PHD LECTURE ON PANEL DATA AT 
#SGH WARSAW SCHOOL OF ECONOMICS. THESE CODES ARE ADOPTED FROM THE BOOK: 
#PANEL DATA ECONOMETRICS WITH R BY YVES CROISSANT & GIOVANNI MILLO, WILEY, 2019.
#COMMENTS ARE MOSTLY BASED ON THIS BOOK AND PROVIDED BY KAROL SZAFRANEK.
#23.10.2022.

##B. NECESSARY R LIBRARIES FOR THIS COURCE--------------------------------------

#LOAD LIBRARIES USED DURING THE COURSE
library(plm)           #PANEL DATA MODELS
library(pder)          #HERE USED FOR LOADING PANEL DATASETS
library(car)           #TESTING FOR APPLIED REGRESSION
library(lmtest)        #LINEAR HYPOTHESES
library(sandwich)      #ROBUST ESTIMATORS
library(splm)          #HERE USED FOR LOADING PANEL DATASETS
library(pglm)          #HERE USED FOR LOADING PANEL DATASETS
library(stargazer)     #PRINT NICELY FORMATTED TABLES
library(texreg)        #PRINT NICELY FORMATTED TABLES
library(ggplot2)       #PLOT NICELY FORMATTED FIGURES

##C. CONTENT--------------------------------------------------------------------

#THE PRESENTED CODE MOSTLY REPLICATES PANEL DATA MODELS FROM WELL KNOWN STUDIES.

#THE FOLLOWING ASPECTS INCLUDE:
#1. THE IMPORTANCE OF UNOBSERVED HETEROGENEITY
#2. ONE-WAY AND TWO-WAY ERROR COMPONENT MODELS
#3. UNBALANCED SAMPLES IN PANEL ESTIMATION
#4. SPECIFICATION TESTS FOR STATIC ERROR COMPONENT PANEL MODELS
#5. ROBUST INFERENCE IN THE PRESENCE OF NON-SPHERICAL VCV MATRIX
#6. ENDOGENEITY AND ESTIMATION OF PANEL INSTRUMENTAL VARIABLES MODELS
#7. ESTIMATION AND TESTING OF DYNAMIC PANEL MODELS

##1. THE IMPORTANCE OF UNOBSERVED HETEROGENEITY---------------------------------

##1.1 INTRODUCTION==============================================================
#WE ALREADY KNOW THAT PANEL DATA HAVE ADVANTAGES OVER PURE CROSS-SECTION AND
#TIME SERIES MODELS. FROM OUR PERSPECTIVE THE CRUCIAL ASPECT IS THAT THEY ALLOW
#TO CONTROL FOR UNOBSERVED VARIABLES POSSIBLY BIASING THE ESTIMATION.
#WE WILL ILLUSTRATE THIS ISSUE USING THE FOLLOWING SIMPLE EXAMPLE.

##1.2 RESEARCH QUESTION: CAN TAXING ALCOHOLICS REDUCE ROAD'S DEATH TOLL?========
#WE MAKE USE THE DATASET FROM STOCK AND WATSON (2007). THE DATASET CONTAINS INFO
#ON YEARS 1982-1988 FOR EACH CONTINENTAL US STATE.
rm(list=ls())                                                                   #CLEAR WORKSPACE
data("Fatalities", package="AER")                                               #IMPORT DATA
?AER::Fatalities                                                                #VIEW DATASET DESCRIPTION
Fatalities$frate <- with(Fatalities, fatal / pop * 10000)                       #CALCULATE FATALITY RATE
fm <- frate ~ beertax                                                           #ESTABLISH MODEL FORMULA
                    
#LET'S START WITH THE OLS                    
mod82 <- lm(fm, Fatalities, subset = year == 1982)                              #CHECK FOR SIGNIFICANCE IN SIMPLE CROSS-SECTION ANALYSIS FOR A GIVEN YEAR
summary(mod82)                                                                  #DISPLAY RESULTS OF THE OLS ESTIMATION
coeftest(mod82)                                                                 #PERFORM T TEST
#TAKE NOTE! THE COEFFICIENT IS POSITIVE BUT INSIGNIFICANT.

#WE CAN MAKE USE OF REPEATED OBSERVATIONS FOR THE SAME INDIVIDUALS.
poolmod <- plm(fm, Fatalities, model="pooling")                                 #ESTIMATE POOLING PANEL MODEL
coeftest(poolmod)                                                               #PERFORM T TEST
#TAKE NOTE! POOLING DATA ONLY MAKES THINGS WORSE. 

#WE SENSE THAT THERE IS SOMETHING WRONG.
#LET'S GET RID OF UNOBSERVED STATE-LEVEL CHARACTERISTICS.
dmod <- plm(diff(frate, 5) ~ diff(beertax, 5), Fatalities, model="pooling")     #ESTIMATE THE MODEL ON 5-YEAR DIFFERENCES
coef(dmod)                                                                      #CHECK COEFFICIENTS
summary(dmod)                                                                   #SUMMARIZE THE MODEL
#TAKE NOTE! CONTROLLING FOR UNOBSERVED CHARACTERISTICS CHANGES OUR INFERENCE.

#WE AlSO CAN CONTROL FOR THEM IN OUR OLS REGRESSION
lsdv.fm <- update(fm, . ~ . + state - 1)                                        #CONTROL FOR THE SAME ASPECTS IN OLS
lsdvmod <- lm(lsdv.fm, Fatalities)                                              #ESTIMATE THE MODEL
coef(lsdvmod)                                                                   #CHECK COEFFICIENTS
summary(lsdvmod)                                                                #SUMMARIZE THE MODEL
#TAKE NOTE! WE HAVE A DIFFERENT ESTIMATE DUE TO DIFFERENT SPECIFICATION.
#HOWEVER, THE CONCLUSION DOES NOT CHANGE>

#WE CAN ALSO DO THIS IN A PANEL SETTING BY ESTABLISHING FIXED EFFECTS,
#WITH MINIMAL ASSUMPTIONS ON THE MODEL HETEROGENEITY.
femod <- plm(fm, Fatalities)                                                    #WG ESTIMATOR IS THE DEFAULT ONE
coeftest(femod)                                                                 #ESTIMATE THE WITHIN MODEL
#TAKE NOTE! WE SEE THAT OUR ESTIMATIONS ARE IN LINE WITH INTUITION.

#NOW WE CAN TEST SOME LINEAR RESTRICTIONS (MORE ON THAT LATER).
lht(femod, "beertax = -1")                                                      #DEFINE LINEAR RESTRICTION TEST (MORE ON THAT LATER)
lht(femod, "beertax = -1",vcov=vcovHC)                                          #ROBUSTIFY THE TEST (MORE ON THAT LATER)
#CONTROLLING FOR THE CONSISTENCY IN VCV MATRIX ESTIMATION CHANGES OUR BELIEFS.
#END OF EXAMPLE

##2. ONE-WAY AND TWO-WAY ERROR COMPONENT MODELS---------------------------------

##2.1 INTRODUCTION==============================================================
##ERROR COMPONENT MODELS ARE RELEVANT WHEN SLOPES ARE SAME FOR ALL INVIDIVUALS.
##INSTEAD, INTERCEPTS ARE A PRIORI DIFFERENT.
##WE WILL DISCUSS HOW TO CONSIDER THE INDIVIDUAL COMPONENT AS A PART OF THE 
##ERROR TERM, WE WILL FOCUS ON POOLING, FIXED EFFECTS AND RANDOM EFFECTS.

##2.2. RESEARCH QUESTION: IS THE INVESTMENT THEORY SUPPORTED BY DATA?===========
##WE USE DATA FROM 188 AMERICAN FIRMS FOR 35 YEARS (1951-1985).
##INVESTMENT IS THE RATIO BETWEEN THE VALUE OF THE FIRM AND THE REPLACEMENT OF 
##ITS PHYSICAL COST OF CAPITAL (TOBIN'S Q). IF THE FINANCIAL MARKET IS PERFECT,
##THEN THE VALUE OF THE FIRM EQUALS THE ACTUAL VALUE OF ITS FUTURE PROFITS.
##FOR TOBIN'S Q>1 THE PROFITABILITY OF THE INVESTMENT IS GREATER THAN ITS COST 
##SO THE INVESTMENT IS VALUABLE.
rm(list=ls())
data("TobinQ", package = "pder")                                                #LOAD DATA
?pder::TobinQ                                                                   #VIEW DATASET DESCRIPTION
pTobinQ  <- pdata.frame(TobinQ)                                                 #DESCRIBE AS PANEL
pTobinQa <- pdata.frame(TobinQ, index = 188)                                    #DESCRIBE AS PANEL
pTobinQb <- pdata.frame(TobinQ, index = c('cusip'))                             #DESCRIBE AS PANEL
pTobinQc <- pdata.frame(TobinQ, index = c('cusip', 'year'))                     #DESCRIBE AS PANEL

summary(pTobinQ)                                                                #SUMMARIZE THE DATA
pdim(pTobinQ)                                                                   #CHECK DIMENSIONS

Qeq <- ikn ~ qn                                                                 #ESTABLISH THE FORMULA
Q.pooling <- plm(Qeq, pTobinQ, model = "pooling")                               #ESTIMATE POOLED OLS
Q.within  <- update(Q.pooling, model = "within")                                #ESTIMATE WITHIN GROUP
Q.between <- update(Q.pooling, model = "between")                               #ESTIMATE BETWEEN GROUP

summary(Q.within)                                                               #DISPLAY ESTIMATION RESULTS
fixef(Q.within,type="level")                                                    #RECOVER FIXED EFFECTS, OTHER TYPES DFIRST OR MEAN
lm(ikn ~ qn + factor(cusip), pTobinQ)                                           #CALCULATE LSDV EQUIVALENT

Q.swar <- plm(Qeq, pTobinQ,                                                     #ESTIMATE A RE MODEL
              model = "random",                                                 #THIS IS RE MODEL
              random.method = "swar")                                           #THIS IS PARTICULAR ESTIMATOR
Q.swar2 <- plm(Qeq, pTobinQ,                                                    #ESTIMATE A DIFFERENT RE MODEL
               model = "random",                                                #THIS IS RE MODEL
               random.models = c("within", "between"),                          #AUXILIARY MODELS TO ESTIMATE VARIANCES
               random.dfcor = c(2, 2))                                          #HOW TO ADJUST THE NUMBER OF PARAMETERS

#TAKE NOTE! FOR FE THE INDIVIDUAL EFFECTS ARE TRANSFORMED OUT, WHILE LSDV 
#CONSIDERS THEM AS PARAMETERS TO BE ESTIMATED. INSTEAD, GLS TREATS THE INDIVIDUAL 
#EFFECTS AS RANDOM DRAWS FROM SPECIFIC DISTRIBUTION. WE ESTIMATE THE PARAMETERS 
#OF THIS DISTRIBUTION TO OBTAIN EFFICIENT ESTIMATORS OF THE SLOPES.

#WHAT PART OF THE VARIANCE IS DUE TO INDIVIDUAL EFFECT - LOOK AT INDIVIDUAL EFFECTS (25%).
#HOW MUCH IS REMOVED FROM EACH VARIABLE - LOOK AT THETA. IS RE CLOSER TO FE OR OLS?
#GLS IS CLOSER TO WG THAN OLS.
summary(Q.swar)                                                                 #ESTIMATION INFO
ercomp(Q.swar)                                                                  #RETAIN SOME ESTIMATION INFO
Q.walhus  <- update(Q.swar, random.method = "swar")                             #ROBUSTNESS CHECK: SWAR
Q.amemiya <- update(Q.swar, random.method = "amemiya")                          #ROBUSTNESS CHECK: AMEMIYA
Q.nerlove <- update(Q.swar, random.method = "nerlove")                          #ROBUSTNESS CHECK: NERLOVE
Q.models <- list(swar = Q.swar,                                                 #COLLECT RESULTS
                 walhus = Q.walhus,                 
                 amemiya = Q.amemiya, 
                 nerlove = Q.nerlove)
sapply(Q.models, function(x) ercomp(x)$theta)                                   #RECOVER THETA
sapply(Q.models, coef)                                                          #RECOVER ESTIMATES
#TAKE NOTE! DIFFERENT GLS ESTIMATORS YIELD SAME RESULTS, POINTING TO ROBUSTNESS.

#LET'S ASK THE QUESTION NOW: FIXED EFFECTS OR RANDOM EFFECTS? 
#BEFORE STATISTICAL TESTING WE SHOULD HAVE INTUITION.
#INDIVIDUAL EFFECTS ARE NOT FIXED OR RANDOM BY NATURE.
#THEY CAN BE TREATED EITHER AS A VECTOR OF CONSTANT PARAMETERS 
#OR REALIZATION OF RANDOM DEVIATES.
#FOR MICRO PANELS RE SEEM APPEALING - DIFFERENT INDIVIDUALS DRAWN FROM LARGE POPULATION.
#FOR MACRO PANELS THE SAMPLE IS FIXED - FE RESEMBLE E.G. COUNTRY DIFFERENCES.

#WHAT DO WE SEE HERE?
#IS OLS MOST EFFICIENT? IT SEEMS SO, 
#BUT IT WILL BE BIASED IF INDIVIDUAL EFFECTS ARE PRESENT.
#WHAT IS THE RELATION BETWEEN GLS AND WG? 
#GLS VARIANCE IS LOWER THAN WG AND BE
sapply(list(pooling = Q.pooling, within = Q.within, 
            between = Q.between, swar = Q.swar),
       function(x) coef(summary(x))["qn", c("Estimate", "Std. Error")])

#HOW CAN BE THE VARIATION OF QN DECOMPOSED?
summary(pTobinQ$qn)                                                             #COMPUTE THE SHARE OF VARIANCE FOR QN

#NOW LET'S FORMALLY STUDY THE RELATION BETWEEN DIFFERENT ESTIMATORS.
#WE CAN USE THE WITHIN AND BETWEEN TRANSFORMATION TO COMPUTE THE WEIGHTS OF WITHING AND BETWEEN ESTIMATORS IN THE OLS
SxxW <- sum(Within(pTobinQ$qn) ^ 2)                                             #THIS IS WITHIN VARIATION
SxxB <- sum((Between(pTobinQ$qn) - mean(pTobinQ$qn)) ^ 2)                       #THIS IS BETWEEN VARIATION
SxxTot <- sum( (pTobinQ$qn - mean(pTobinQ$qn)) ^ 2)                             #THIS IS TOTAL VARIATION
pondW <- SxxW / SxxTot                                                          #THIS IS RATIO WITHIN/TOTAL
pondW                                                                           #THE WEIGHT OF THE WITHING IS 57%
pondW * coef(Q.within)[["qn"]] + (1 - pondW) * coef(Q.between)[["qn"]]          #OLS IS HALFWAY BETWEEN WITHIN AND BETWEEN

T <- 35                                                                         #T DIMENSION
N <- 188                                                                        #N DIMENSION 
smxt2 <- deviance(Q.between) * T / (N - 2)                                      #TO ESTIMATE THETA
sidios2 <- deviance(Q.within) / (N * (T - 1) - 1)                               #TO ESTIMATE THETA 
phi <- sqrt(sidios2 / smxt2)                                                    #TO ESTIMATE THETA
pondW <- SxxW / (SxxW + phi^2 * SxxB)                                           #THIS IS THE SHARE
pondW                                                                           #THE WEIGHT OF THE WITHIN IS 95%
pondW * coef(Q.within)[["qn"]] +(1 - pondW) * coef(Q.between)[["qn"]]           #THIS IS DUE TO THE LARGE T 

Q.models2 <- lapply(Q.models, function(x) update(x, effect = "twoways"))        #THIS MODEL ESTIMATES TWO ERROR COMPONENT MODEL
sapply(Q.models2, function(x) sqrt(ercomp(x)$sigma2))                           #ESTIMATES OF THE VARIANCE COMPONENTS ARE SIMILAR
sapply(Q.models2, function(x) ercomp(x)$theta)                                  #75% OF INDIVIDUAL AND TIME MEANS ARE REMOVED FROM VARIABLES

##2.3. RESEARCH QUESTION: WHAT ARE THE RETURNS TO EDUCATION?====================
#NOT LET'S DISCUSS THE CHOICE OF THE ESTIMATOR FROM A DIFFERENT PERSPECTIVE.
#LOOK HOW THE CHOICE OF THE ESTIMATOR AFFECTS OUR RESULTS. 
#CONSIDER THE DATASET BY BELLA AND VERBEEK (1998) TO STUDY THE IMPACT OF UNION
#NEGOTIATIONS ON WAGES. WE HAVE DATA FOR 545 MEN DURING 8 YEARS (1980-1987)
rm(list=ls())
data("UnionWage", package = "pglm")                                             #LOAD DATA
?pglm::UnionWage                                                                #VIEW DATASET DESCRIPTION
pdim(UnionWage)                                                                 #CHECK DIMENSIONS
summary(UnionWage)                                                              #SUMMARIZE DATA
UnionWage$exper2 <- with(UnionWage, exper ^ 2)                                  #MANIPULATE DATA

wages.within1 <- plm(wage ~ union + school + exper + exper2 +                   #WG MODEL 1
                       com + rural + married + health +
                       region + sector, UnionWage)
wages.within2 <- plm(wage ~ union + school + exper + exper2 +                   #WG MODEL 2
                       com + rural + married + health +
                       region + sector + occ, UnionWage)
wages.pooling1 <- update(wages.within1, model = "pooling")                      #POOLING MODEL 1
wages.pooling2 <- update(wages.within2, model = "pooling")                      #POOLING MODEL 2

#LET'S GATHER ALL RESULTS. THIS IS AN EXTREMELY USEFUL TOOL BUT WE WON'T DISCUSS
#ITS FUNCTIONALITY AT LENGTH
stargazer(wages.pooling2,
          wages.pooling1, 
          wages.within2, 
          wages.within1,
          type="text",
          omit = c("region", "sector", "occ"),
          omit.labels = c("region dummies", "sector dummies", "occupation dummies"),
          column.labels = c("pooling estimation", "within estimation"),
          column.separate = c(2, 2),
          dep.var.labels = "log of hourly wage",
          covariate.labels = c("union membership", "education years",
                               "experience years", "experience years squared",
                               "black", "hispanic", "rural residence",
                               "married", "health problems",
                               "Intercept"),
          omit.stat = c("adj.rsq", "f"),
          title = "Wage equation",
          label = "tab:wagesresult",
          no.space = TRUE
)

#WHAT DO YOU SEE? 
#UNION PREMIUM FALLS WITH WG ESTIMATOR. SO WE KNOW THAT INVIDUAL EFFECTS 
#ARE STRONGLY CORRELATED WITH THIS VARIABLE. NEXT, RETURN FOR EDUCATION IS 
#AROUND 8% FOR ONE MORE YEAR OF EDUCATION. THIS WILL BE CONSISTENT IF EDUCATION
#IS NOT CORRELATED WITH INDIVIDUAL EFFECT. HOWEVER, WG ELIMINATES THESE EFFECTS 
#FROM ESTIMATION. WE KNOW NOW THAT WG IS NOT THE TOOL WE WANT TO USE FOR SUCH 
#DATASET WHEN TEMPORAL VARIATION IS LOW OR NONE.

#TO SUM UP:
#1. NO CORRELATION BETWEEN INDIVIDUAL EFFECTS AND REGRESSORS?
#THEN USE GLS, IT GIVES CONSISTENT AND EFFICIENT ESTIMATORS.
#2. PRESENT CORRELATION BETWEEN INDIVIDUAL EFFECTS AND REGRESSORS?
#THEN USE WITHIN, BUT REMEMBER THAT IT ELIMINATES TIME-INVARIANT VARIABLES.
#3. IF THERE IS CORRELATION BETWEEN COVARIATES AND BOTH ERROR TERM,
#THEN DON'T USE EITHER RE OR FE, BOTH ESTIMATORS ARE INCONSISTENT.

##3. UNBALANCED SAMPLES IN PANEL ESTIMATION-------------------------------------

##3.1 INTRODUCTION==============================================================
#FOR UNBALANCED PANELS THE KEY FEATURE IS THAT NOW THE NUMBER OF OBSERVATIONS IS
#INDIVIDUAL FOR SPECIFIC UNIT. IT COMPLICATES THE FORMULATION OF THE COVARIANCE 
#MATRIX. IT ALSO COMPLICATES THE CALCULATION OF INDIVIDUAL EFFECTS MODEL.
#THERE IS NO OBVIOUS CHOICE FOR DENOMINATORS TO ACCOUNT FOR THE SIZE AND CORRECT
#FOR DEGREES OF FREEDOM.

##3.2 RESEARCH QUESTION: WHAT ARE THE PRODUCTION FUNCTION PARAMETERS?===========
#WE ESTABLISH  COBB-DOUGLAS  PRODUCTION FUNCTION FOR THE DATA ON 25 FIRMS 
#FROM 1982 TO 1983. THESE ARE WEEKLY OBSERVATIONS
#THE DATASET CONCERNS THE PRODUCTION OF TILERIES IN EGYPT
#IN CD PRODUCTION FUNCTION OUTPUT DEPENDS ON QUANTITY OF INPUTS, HERE TWO:
#LABOR AND MACHINES. WE ESTIMATE THE FE AND RE MODEL.
rm(list=ls())
data("Tileries", package = "pder")                                              #IMPORT DATA
?pder::Tileries                                                                 #VIEW DATASET DESCRIPTION
head(Tileries, 3)                                                               #CHECK FIRST ROWS 
pdim(Tileries)                                                                  #CHECK DIMENSIONS
Tileries <- pdata.frame(Tileries)                                               #WRITE AS PANEL DATA FRAME
plm.within <- plm(log(output) ~ log(labor) + log(machine), Tileries)            #ESTIMATE FE MODEL
tile.r <- plm(log(output) ~ log(labor) + log(machine), Tileries,                #ESTIMATE RE MODEL
              model = "random")                                            
summary(tile.r)

#TAKE NOTE! THE TRANSFORMATION PARAMETERS IS NOW INDIVIDUAL-SPECIFIC.
#NOW LOOK AT THE TWO-WAY ERROR COMPONENT MODEL.
plm.within <- plm(log(output) ~ log(labor) + log(machine),Tileries, 
                  effect = "twoways")                                           #ESTIMATE TWO-WAYS COMPONENT MODEL
lm.lsdv <- lm(log(output) ~ log(labor) + log(machine) + 
                factor(id) + factor(week), Tileries)                            #SAME AS LSDV
wh <- plm(log(output) ~ log(labor) + log(machine), Tileries,                    #NOW CHOOSE RE WALHUS
          model = "random", random.method = "walhus",
          effect = "twoways")
am <- update(wh, random.method = "amemiya")                                     #UPDATE SPECIFICATION TO AMEMIYA
sa <- update(wh, random.method = "swar")                                        #AND TO SWAR
ercomp(sa)                                                                      #LOOK AT THE SHARES 
#TAKE NOTE! THE SHARES FOR THE INDIVIDUAL AND TIME EFFECTS ARE 19% AND 5%, RESPECTIVELY.

#NOW COMPARE MODELS
re.models <- list(walhus = wh, amemiya = am, swar = sa)                         #DEFINE LIST
sapply(re.models, function(x) sqrt(ercomp(x)$sigma2))                           #REPORT SIGMAS
sapply(re.models, coef)                                                         #REPORT COEFFICIENTS

##4. SPECIFICATION TESTS FOR STATIC ERROR COMPONENT PANEL MODELS----------------

##4.1 INTRODUCTION==============================================================
#DUE TO DIMENSIONALITY WE CAN HAVE MUCH RICHER SPECIFICATIONS. HENCE WE SHOULD
#DIAGNOSE THESE SPECIFICATIONS ACCORDINGLY. SPECIFICATION PROBLEM OF PANEL 
#MODELS IS ASSOCIATED WITH THE PRESENCE OF ABSENCE OF INDIVIDUAL EFFECTS.
#WE WILL CHECK WHETHER THE INVIDUAL EFFECTS ARE CORRELATED OR UNCORRELATED WITH 
#THE EXPLANATORY VARIABLES. NEXT QUESTION IS HOW REMAINDER ERRORS ARE 
#CORRELATED: ARE THEY CORRELATED IN TIME OR IN SPACE OR IN SOME SPACE.
#FOR LINEAR PANEL MODELS THERE IS A RICH TOOLBOX FOR DIAGNOSTICS.

##4.2 RESEARCH QUESTION: WHAT DRIVES THE OUTPUT OF RICE FARMS?==================
##WE ESTIMATE AGAIN A COBB-DOUGLAS FUNCTION. THE DATASET CONCERNS THE PRODUCTION
#OF RISE IN INDIA. WE MAKE USE OF A PANEL OF N=171 OBSERVATIONS FOR T=6 TIME
#PERIODS. 
rm(list=ls())
data("RiceFarms", package = "splm")                                             #LOAD DATA
?splm::RiceFarms                                                                #VIEW DATASET DESCRIPTION
Rice <- pdata.frame(RiceFarms, index = "id")                                    #PANEL DATA FRAME
rice.w <- plm(log(goutput) ~ log(seed) + log(totlabor) + log(size), Rice)       #WITHIN MODEL
rice.p <- update(rice.w, model = "pooling")                                     #POOLED MODEL
rice.wd <- plm(log(goutput) ~ log(seed) + log(totlabor) + log(size), Rice,      #FE AND TIME EFFECTS
               effect = "twoways") 

##4.2.1 MODEL SPECIFICATION: SIGNIFICANCE OF INDIVIDUAL EFFECTS#################
#FIRST, WE RUN SIMPLE F TESTS, USED TO COMPARE NESTED MODELS. 
pFtest(rice.w, rice.p)                                                          #TEST FOR INDIVIDUAL EFFECTS
pFtest(log(goutput) ~ log(seed) + log(totlabor) + log(size), Rice)              #ALTERNATIVELY THE SAME
pFtest(rice.wd, rice.p)                                                         #TEST FOR ALL EFFECTS
pFtest(log(goutput) ~ log(seed) + log(totlabor) + log(size), Rice,              #ALTERNATIVELY THE SAME
       effect = "twoways")
pFtest(rice.wd, rice.w)                                                         #TEST FOR TIME EFFECTS
#TAKE NOTE! WE SEE THAT ALL THESE EFFECTS ARE STRONGLY REJECTED.

#NEXT, WE RUN BREUSCH-PAGAN LAGRANGE MULTIPLIER TESTS BASED ON RESIDUALS.
plmtest(rice.p)                                                                 #SUPPLY OLS MODEL
plmtest(log(goutput)~log(seed)+log(totlabor)+log(size), Rice)                   #OR FORMULA
plmtest(rice.p, effect = "time")                                                #TEST FOR SIGNIFICANCE OF TIME EFFECTS
plmtest(rice.p, effect = "twoways")                                             #HERE BOTH
plmtest(rice.p, effect = "twoways", type = "kw")                                #USE HONDA (KING WU) 
plmtest(rice.p, effect = "twoways", type = "ghm")                               #OR GOURIEROUX MODIFICATIONS FOR UNBALANCED PANELS

##4.2.2 MODEL SPECIFICATION: HAUSMAN-TYPE TEST##################################
#WE RETURN TO ONE OF THE MOST IMPORTANT QUESTIONS: SHOULD WE USE FIXED OR RANDOM
#EFFECTS? REMEMBER, IF THERE IS NO CORRELATION OF X WITH INDIVIDUAL EFFECTS: 
#BOTH RE AND FE CONSISTENT, BUT GLS WILL BE EFFICIENT. HOWEVER, IF THERE IS
#CORRELATION OF X WIHT INDIVIDUAL EFFECTS: FE CONSISTENT, WHILE GLS NOT.

#WE'LL MAKE USE OF THE HAUSMAN TEST. IT IS CONSTRUCTED AS FOLLOWS:
#H0: A & B ARE CONSISTENT BUT B IS MORE EFFICIENT THAN A
#H1: ONLY A IS CONSISTENT.
rice.w <- plm(log(goutput) ~ log(seed) + log(totlabor) + log(size), Rice)       #WG ESTIMATOR
rice.r <- update(rice.w, model = "random")                                      #GLS ESTIMATOR
phtest(rice.w, rice.r)                                                          #HAUSMAN TEST
#TAKE NOTE! WE DO NOT REJECT THE NULL. THE TEST POINTS TO RE MODEL.

#NOW WE MAKE USE OF THE MUNDLAK (1978) APPROACH.
rice.b <- update(rice.w, model = "between")                                     #ESTIMATE BETWEEN MODEL
cp <- intersect(names(coef(rice.b)), names(coef(rice.w)))                       #MATCH VARIABLES
dcoef <- coef(rice.w)[cp] - coef(rice.b)[cp]                                    #CALCULATE DIFFERENCE BETWEEN WITHIN AND BETWEEN
V <- vcov(rice.w)[cp, cp] + vcov(rice.b)[cp, cp]                                #CALCULATE THE VCV MATRIX 
as.numeric(t(dcoef) %*% solve(V) %*% dcoef)                                     #CALCULATE THE CHI SQUARE
cor(fixef(rice.w), between(log(Rice$goutput)))                                  #CALCUALTE THE CORRELATION BETWEEN FE AND INDIVIDUAL MEANS
#TAKE NOTE! WE GET THE SAME RESULT.

#NOW WE CONDUCT THE CHAMBERLAIN TEST TO BE ENTIRELY SURE. 
piest(log(goutput) ~ log(seed) + log(totlabor) + log(size),                     #DECLARE FORMULA
      RiceFarms, index = "id")
aneweytest(log(goutput) ~ log(seed) + log(totlabor) + log(size),                #DECLARE FORMULA
           RiceFarms, index = "id")
#TAKE NOTE! RESTRICTION IMPLIED BY THE WITHIN MODEL ARE REJECTED AT 5% SIG LEVEL

##4.2.3 MODEL SPECIFICATION: SERIAL AUTOCORRELATION#############################
#WE NOW TURN TO TESTS FOR SERIAL CORRELATION.
#MODEL WITH INDIVIDUAL EFFECTS HAS COMPOSITE ERRORS SERIALLY CORRELATED BY 
#DEFINITION. THIS IS DUE TO THE TIME-INVARIANT ERROR COMPONENT THAT INDUCES 
#SERIAL CORRELATION. THIS EFFECT DOES NOT DIE IN TIME. HENCE, BY SERIAL 
#AUTOCORRELATION WE MEAN TIME-DECAYING TYPE IN THE IDIOSYNCRARTIC ERROR TERM.

#WE CAN USE MARGINAL, JOINT OR CONDITIONAL TEST OF AUTOCORRELATION, ALSO SEMI-
#PARAMETRIC ALTERNATIVES, ROBUST HC VERSIONS AND FREE FROM DISTRIBUTIONAL ASSUMPTIONS
fm <- log(goutput) ~ log(seed) + log(totlabor) + log(size)                      #FORMULA

#FIRST WE RUN UNOBSERVED EFFECTS TEST A LA WOOLDRIDGE (SEMI-PARAMETRIC).
#IT CHECKS WHETHER THERE ARE NO UNOBSERVED EFFECTS IN THE RESIDUALS - THE 
#COVARIANCE MATRIX OF THE RESIDUALS SHOULD BE THUS DIAGONAL FOR EACH I.
#NOT REJECTING FAVOURS OLS, REJECTION MAY BE DUE TO SERIAL CORRELATION OF 
#DIFFERENT KINDS, BUT THE TEST IS MUTE ON WHERE TO GO.
pwtest(fm, Rice)                                           

#SECOND, WE CAN RUN THE BALTAGI AND LI JOINT TEST OF NO SERIAL CORRELATION AND 
#NO INDIVIDUAL EFFECTS, OR THE MARGINAL TEST OF SERIAL CORRELATION WITH 
#MAINTAINED HYPOTHESIS OF ABSENCE OF INDIVIDUAL EFFECTS, OR THE CONDITIONAL TEST
#WITH MAINTAINED HYPOTHESIS OF PRESENCE OF INDIVIDUAL EFFECTS.

#LM TESTS FOR RANDOM EFFECTS AND/OR SERIAL CORRELATION
#J = JOINT TEST FOR RANDOM EFFECTS AND SERIAL CORRELATION UNDER NORMALITY AND HOMOSCEDASTICITY 
#RE OR AR = LOCALLY ROBUST TEST PROVIDING THE EVIDENCE ABOUT THE DIRECTION OF MISSPECIFICATION
bsy.LM <- matrix(ncol=3, nrow = 2)                                              #ALLOCATE SPACE
tests <- c("J", "RE", "AR")                                                     #ESTABLISH TESTS
dimnames(bsy.LM) <- list(c("LM test", "p-value"), tests)                        #AESTHETHICS
for(i in tests) {                                                               #RUN SEQUENTIALLY
  mytest <- pbsytest(fm, data = Rice, test = i)
  bsy.LM[1:2, i] <- c(mytest$statistic, mytest$p.value)
}
round(bsy.LM, 6)                                                                #COMPARE RESULTS
#TAKE NOTE! WE DIFFERENTIATE BETWEEN TIME-INVARIANT PERSISTENCE AND 
#TIME-DECAYING PERSISTENCE. WE CONCLUDE THAT THERE IS PROBLEM OF THE SECOND KIND.

#CONDITIONAL TEST OF BALTAGI AND LI FOR SERIAL CORRELATION ALLOWING FOR RANDOM EFFECTS
#OF ANY MAGNITUTE
pbltest(fm, Rice, alternative = "onesided")

#FINALLY, WE CAN RUN TRADITIONAL SERIAL CORRELATION TESTS A LA BREUSCH-GODFREY
#OR DURBIN WATSON
rice.re <- plm(fm, Rice, model='random')
pbgtest(rice.re, order = 2)                                                     #BREUSCH-GODFREY 
pdwtest(rice.re, order = 2)                                                     #DURBIN-WATSON TESTS
#TAKE NOTE! WE HAVE PROBLEMS WITH SERIAL AUTOCORRELATION IRRESPECTIVE OF THE TEST.

#REMEMBER THAT IN FE MODELS RESIDUALS THERE IS SERIAL CORRELATION DUE TO WITHIN
#TRANSFORMATION. IT DIES AS T INCREASES. AUTOCORRELATION TESTS ARE BASED ON 
#T-ASYMPTOTICS, THEREFORE IN SHORT PANELS THEY ARE SEVERELY BIASED TOWARDS REJECTION.

#WE WILL PERFORM THE WOOLDRIDGE WITHIN-BASED TEST FOR SHORT PANEL. IT RUNS 
#POOLED REGRESSION OF FE RESIDUALS ON THEIR FIRST LAG. TO ILLUSTRATE HOW THIS 
#TEST WORK WE MAKE USE OF THE DATASET ON EMPLOYMENT AND WAGES IN THE UK. 
#IT IS AN UNBALANCED PANEL OF 140 OBSERVATIONS FROM 1976 TO 1984.
data("EmplUK", package = "plm")                                                 #WE NEED LARGE SHORT PANEL
?plm::EmplUK                                                                    #VIEW DATASET DESCRIPTION
pwartest(log(emp) ~ log(wage) + log(capital), data = EmplUK)                    #CARRY OUT THE TEST

#WOOLDRIDGE FIRST DIFFERENCE-BASED TEST
#SPECIFICATION TEST TO CHOOSE THE MOST EFFICIENT ESTIMATOR BETWEEN FE AND FD
pwfdtest(log(emp) ~ log(wage) + log(capital), data = EmplUK)                    #TEST FOR SERIAL CORRELATION IN FIRST-DIFFERENCED ERRORS
pwfdtest(log(emp) ~ log(wage) + log(capital), data = EmplUK,h0 = "fe")          #TEST FOR NO SERIAL CORRELATION IN OROGINAL ERRORS, SIMILAR TO PWAR TEST

#TAKE NOTE! ORIGINAL RESIDUALS SHOW EVIDENCE ON SERIAL CORRELATION.
#IT POINTS TO UNIT ROOT IN THE ERRORS.

#WE NOW COME BACK TO THE ORIGINAL EXAMPLE AND CONDUCT THE WOOLDRIDGE FIRST 
#DIFFERENCE TEST FOR THE RICE FARMS DATASET
W.fd <- matrix(ncol = 2, nrow =2)                                               #ALLOCATE SPACE
H0 <- c("fd", "fe")                                                             #H0 TESTED
dimnames(W.fd) <- list(c("test", "p-value"), H0)                                #AESTHETICS
for(i in H0) {                                                                  #CONDUCT TESTS ITERATEIVELY
  mytest <- pwfdtest(fm, Rice, h0 = i)
  W.fd[1, i] <- mytest$statistic
  W.fd[2, i] <- mytest$p.value
}
round(W.fd, 6)                                                                  #PRINT RESULTS
#TAKE NOTE! NO MATTER WHAT WE DO HERE WE ARE LEFT WITH AUTOCORRELATION
#USE HAC CORRECTION FOR THE VARIANCE-COVARIANCE MATRIX

##4.2.4 MODEL SPECIFICATION: CROSS-SECTIONAL DEPENDENCE#########################
#TESTS FOR CROSS-SECTIONAL DEPENDENCE ARE VERY IMPORTANT IN PANEL ASPECTS. CROSS
#SECTIONAL DEPENDENCE MAY BE DUE TO THE FACT THAT INDIVIDUALS MAY REACT TO
#COMMON SHOCKS, OR INDIVIDUALS MAY BE DEPENDEND ON A MEASURE OF DISTANCE.
#CROSS-SECTIONAL DEPENDENCE MAKES ESTIMATORS INEFFICIENT, BUT IF UNOBSERVED 
#FACTORS ARE ALSO CORRELATED WITH REGRESSORS, THAN WE HAVE A MORE SERIOUS 
#PROBLEM: INCONSISTENCY. FOR CROSS-SECTIONAL DEPENDENCE TESTS WE WILL MOSTLY
#RELY ON SOME SORT OF AGGREGATING THE INFORMATION FROM PAIRWISE CORRELATION 
#COEFFICIENTS. IF THE CORRELATION COEFFICIENTS CANCEL EACH OTHER OUT, ABSOLUTE 
#MEASURE IS OFTEN CALCULATED. 

#FAMILY OF CROSS-SECTIONAL DEPENDENCE TESTS CAN BE APPLIED IN DIFFERENT SETTINGS
#BREUSCH-PAGAN LM TEST WITH SQUARED RHO IS VALID FOR T->INFTY WITH N FIXED, BUT
#INAPPROPRIATE IF N->INFTY. USE SCLM IF T->INFTY AND N->INFTY.
#PESARAN CD TEST IS BASED ON RHO WITHOUT SQUARING AND IS APPROPRIATE BOTH FOR
#N AND T ASYMPTOTICS. CROSS-SECTIONAL DEPENDENCE TESTS HAVE GOOD PROPERTIES
#IN VARIOUS SAMPLES OF ANY SIZE AND ARE GENERALLY ROBUST TO A VARIETY OF
#SETTINGS, BUT THEY ARE SUSCEPTIBLE TO POWER LOSS WHEN AVERAGE CORRELATION IS 
#ZERO, E.G. WHEN SOME UNIT REACT POSITIVELY AND OTHERS NEGATIVELY TO A SHOCK.

#TO ILLUSTRATE THESE TESTS WE WILL USE THE CASE OF RETURNS OF OWN R&D IN THE 
#PRODUCTION FUNCTION OF EUROPEAN FIRMS. THIS IS A PANEL DATASET FOR 119 
#INDUSTRIES OVER 1980-2005
data("RDSpillovers", package = "pder")                                          #LOAD DATA
?pder::RDSpillovers                                                             #VIEW DATASET DESCRIPTION
fm.rds <- lny ~ lnl + lnk + lnrd                                                #ESTABLISH FORMULA
pcdtest(fm.rds, RDSpillovers)                                                   #BASIC CD TEST
rds.2fe <- plm(fm.rds, RDSpillovers, model = "within", effect = "twoways")      #COMPLICATE THE MODEL
pcdtest(rds.2fe)                                                                #TEST THE RESIDUAL CROSS-SECTIONAL DEPENDENCE
#TAKE NOTE! NOW WE DO NOT REJECT HYPOTHESIS OF NO CROSS-SECTIONAL CORRELATION

#NOW LET'S CALCULATE THE RHO AND ABS(RHO) TO CHECK WHAT IS GOING ON.
cbind("rho"  = pcdtest(rds.2fe, test = "rho")$statistic,                        #CALCULATE RHO AND ABS(RHO)
      "|rho|"= pcdtest(rds.2fe, test = "absrho")$statistic)
#TAKE NOTE! WE SEE IN FACT A SUBSTANTIAL CROSS-CORRELATION BUT THE ADDITION OF 
#TIME EFFECTS ACCOUNTING FOR A COMMON SHOCK CENTERS THEM ON ZERO.


##5. ROBUST INFERENCE IN THE PRESENCE OF NON-SPHERICAL VCV MATRIX---------------

##5.1 INTRODUCTION==============================================================
#NOW WE WILL RELAX THE ASSUMPTION ON INDEPENDENCE AND HOMOSCEDASTICITY OF THE 
#REMAINDER ERRORS. FOR ROBUST APPROACHES, WE WILL USE APPROPRIATE ESTIMATOR FOR 
#THE PARAMETERS' COVARIANCE MATRIX. ERROR CORRELATION IN PANELS WILL OFTEN 
#DEPEND ON CLUSTERING ISSUES. THEY MAY BE DUE TO COMMON INDIVIDUAL UNIT, 
#TIME PERIOD, ETC. VARIANCE ESTIMATES ARE TYPICALLY BIASED DOWNWARDS UNDER THE 
#ASSUMTION ON RANDOM SAMPLING, SO WE NEED TO ADJUST ACCORDINGLY OUR ESTIMATES OF
#STANDARD ERRORS IN ORDER TO INTERPRET VARIABLES SIGNIFICANCE CORRETLY.

##5.2 RESEARCH QUESTION: HOW PUBLIC INFRASTRUCTURE IMPACTS ECON ACTIVITY?=======
##TO ANSWER IT WE USE A SAMPLE OF 48 US STATES OVER 17 YEARS (1970-1968). WE
#RELY ON THE COBB-DOUGLAS FUNCTION RELATING GROSS SOCIAL PRODUCT TO INPUT OF 
#PUBLIC CAPITAL, PRIVATE CAPITAL AND LABOUR, CONTROLING FOR UNEMPLOYMENT.
rm(list=ls())
data("Produc", package = "plm")                                                 #LOAD DATA
?plm::Produc                                                                    #VIEW DATASET DESCRIPTION
fm <- log(gsp) ~ log(pcap) + log(pc) + log(emp) + unemp                         #ESTABLISH MODEL FORMULA
lmmod <- lm(fm, Produc)                                                         #CALCULATE OLS VERSION
coeftest(lmmod, vcov = vcovHC)                                                  #USE OLS HC VCV MATRIX
plmmod <- plm(fm, Produc, model = "pooling")                                    #CALCULATE PANEL OLS VERSION
summary(plmmod, vcov = vcovHC)                                                  #USE PANEL HC VCV MATRIX, THE SAME WILL BE FOR WHITE1 AND HC3
coeftest(plmmod, vcov=vcovSCC)                                                  #NOW ACCOUNT FOR SERIAL CORRELATION BETWEEN STATES AND ACROSS
coeftest(plmmod, vcov=vcovDC)                                                   #USE DOUBLE CLUSTERING

#TAKE NOTE! NATURALLY, COEFFICIENTS DO NOT CHANGE, BUT STANDARD ERROS INCREASE 
#A LOT. 

#WE NOW BUILD MUCH MORE COMPLICATED VARIANCE-COVARIANCE CORRECTIONS
#LETS ACCOUNT FOR DOUBLE CLUSTERING AND FOUR PERIODS OF PERSISTENT UNWEIGHTED
#SHOCKS.
myvcovDCS <- function(x, maxlag = NULL, ...) {                                  #THIS COMPLICATES THE ISSUE EVEN FURTHER FOR DOUBLE CLUSTERING WITH PERIODS OF PERSISTENT SHOCKS A LA THOMPSON (2011)
  w1 <- function(j, maxlag) 1
  VsccL.1 <- vcovSCC(x, maxlag = maxlag, wj = w1, ...)
  Vcx <- vcovHC(x, cluster = "group", method = "arellano", ...)
  VnwL.1 <- vcovSCC(x, maxlag = maxlag, inner = "white", wj = w1, ...)
  return(VsccL.1 + Vcx - VnwL.1)
}
coeftest(plmmod, vcov=function(x) myvcovDCS(x, maxlag = 4))                     #USE THE DOUBLE CLUSTERING WITH 4 PERIODS OF PERSISTENT UNWEIGHTED SHOCKS

#NOW LETS COMPUTE A VAST ARRAY OF STANDARD ERRORS. LET'S TREAT THIS AS A TYPE
#OF SENSITIVITY ANALYSIS WITH RESPECT TO VARIOUS MISSPECIFICATION PROBLEMS.
Vw <- function(x) vcovHC(x, method = "white1")                                  #STANDARD WHITE 
Vcx <- function(x) vcovHC(x, cluster = "group", method = "arellano")            #GROUP-CLUSTERED
Vct <- function(x) vcovHC(x, cluster = "time", method = "arellano")             #TIME-CLUSTERED
Vcxt <- function(x) Vcx(x) + Vct(x) - Vw(x)                                     #GROUP-TIME CLUSTERED
Vct.L <- function(x) vcovSCC(x, wj = function(j, maxlag) 1)                     #SSC CLUSTERED
Vnw.L <- function(x) vcovNW(x)                                                  #NEWEY WEST
Vscc.L <- function(x) vcovSCC(x)                                                #SSC CLUSTERD
Vcxt.L<- function(x) Vct.L(x) + Vcx(x) - vcovNW(x, wj = function(j, maxlag) 1)  #DOUBLE CLUSTERING WITH TIME LAG
vcovs <- c(vcov, Vw, Vcx, Vct, Vcxt, Vct.L, Vnw.L, Vscc.L, Vcxt.L)              #COLLECT 
names(vcovs) <- c("OLS", "Vw", "Vcx", "Vct", "Vcxt", "Vct.L",                   #NAME ESTIMATORS
                  "Vnw.L","Vscc.L", "Vcxt.L") 
cfrtab <- function(mod, vcovs, ...) {                                           #DECLARE COLLECTING FUNCTION
  cfrtab <- matrix(nrow = length(coef(mod)), ncol = 1 + length(vcovs))
  dimnames(cfrtab) <- list(names(coef(mod)),
                           c("Coefficient", paste("s.e.", names(vcovs))))
  cfrtab[,1] <- coef(mod)
  for(i in 1:length(vcovs)) {
    myvcov = vcovs[[i]]
    cfrtab[ , 1 + i] <- sqrt(diag(myvcov(mod)))
  }
  return(t(round(cfrtab, 4)))
}
cfrtab(plmmod, vcovs)                                                           #COMPARE ALL RESULTS
#TAKE NOTE! WE SEE THE IMPORTANCE OF NON-DECAYING SERIAL ERROR DEPENDENCE.
#CONSEQUENTLY, WE CAN ARGUE  THAT DUE TO THE ADDITIVE NATURE OF V_WH, V_CX AND 
#V_CT WE CAN INFER WHAT IS THE PROBLEM WITHIN OUR FRAMEWORK. 
#E.G. IF V_CX<<C_T~C_CXT THEN THERE IS IMPORTANT CROSS SECTIONAL CORRELATION.
#REMEMBER THAT ROBUST VARIANCE COVARIANCE MATRIX MAY BE INCLUDED IN E.G. LINEAR 
#HYPOTHESIS TESTS OR THE ROBUSTIFIED HAUSMAN REGRESSION-BASED TEST.
linearHypothesis(plmmod,"log(pcap)+log(pc)+log(emp)=1",vcov=vcovSCC)            #ROBUSTIFIED LINEAR HYPOTHESIS TEST

#TO ILLUSTRATE THE ROBUST HAUSMAN TEST LET'S USE THE RD SPILLOVERS EXAMPLE
data("RDSpillovers", package = "pder")                                          #LOAD DATA      
?pder::RDSpillovers                                                             #VIEW DATASET DESCRIPTION
pehs <- pdata.frame(RDSpillovers, index = c("id", "year"))                      #ESTABLISH DATASET
ehsfm <- lny ~ lnl + lnk + lnrd                                                 #WRITE MODEL FORMULA
phtest(ehsfm, pehs)                                                             #STANDARD HAUSMAN
phtest(ehsfm, pehs, method = "aux")                                             #REGRESSION-BASED HAUSMAN
phtest(ehsfm, pehs, method = "aux", vcov = vcovHC)                              #REGRESSION-BASED ROBUST HAUSMAN

##6. ENDOGENEITY AND ESTIMATION OF PANEL INSTRUMENTAL VARIABLES MODELS----------

##6.1 INTRODUCTION==============================================================
#WE KNOW THAT ENDOGENEITY CAN BE PRESENT IN THE MODEL DUE TO SIMULTANEITY,
#MEASUREMENT ERRORS IN COVARIATES OR OMITTED VARIABLES. IN OLS ENDOGENEITY LEADS
#TO INCONSISTENT ESTIMATES. WE CAN THEN USE 2SLS (INSTRUMENTAL VARIABLES 
#ESTIMATOR). IT APPLIES OLS TWICE: WE FIRST REGRESS EVERY COVARIATE ON THE
#INSTRUMENTS AND NEXT WE REGRESS THE RESPONSE ON THE FITTED FIRST-STAGE VALUES.
#IN PANEL DATA ERROR TERM HAS TWO COMPONENTS (INDIVIDUAL AND IDIOSYNCRATIC).
#IN FE MODELS INDIVIDUAL EFFECT IS WIPED OUT SO CORRELATION WITH COVARIATES AND 
#INDIVIDUAL EFFECT NEED NOT BE TAKEN INTO ACCOUNT. FOR RE MODELS THE SITUATION
#IS DIFFERENT.

y ~ x1 + x2 + x3 | x1 + x3 + z                                                  #EQUIVALENT FORMULA NOTATION
y ~ x1 + x2 + x3 | . - x2 + z                                                   #EQUIVALENT FORMULA NOTATION

##6.2 RESEARCH QUESTION: HOW SEATBELTS AFFECT THE NUMBER OF DEATHS?=============
#WE WILL ANSWEAR THIS QUESTION WITH DATA FOR 50 AMERICAN STATES FOR 1983-1997 
#PERIOD (N=51, T=15). WE NOTE THAT: FIRST, USING DRIVERS MAKES THE DRIVER MORE 
#CONFIDENT AND LESS PRUDENTLY DRIVING. SECOND, IF DRIVING CONDITIONS WORSEN THEN
#ROAD MORTALITY INCREASES BUT SEAT BELT USE INCREASES AS WELL
rm(list=ls())
data("SeatBelt", package = "pder")                                              #LOAD DATA
?pder::SeatBelt                                                                 #VIEW DATASET DESCRIPTION
SeatBelt$occfat <- with(SeatBelt, log(farsocc / (vmtrural + vmturban)))         #THE NUMBER OF VEHICLE OCCUPANTS KILLED ON ROAD
ols <- plm(occfat ~ log(usage) + log(percapin) + log(unemp) + log(meanage) +    #OLS POOLED REGRESSION
             log(precentb) + log(precenth)+ log(densrur) + 
             log(densurb) + log(viopcap) + log(proppcap) +
             log(vmtrural) + log(vmturban) + log(fueltax) +
             lim65 + lim70p + mlda21 + bac08, SeatBelt, 
           effect = "time")
fe <- update(ols, effect = "twoways")                                           #FE MODEL ESTIMATION
ivfe <- update(fe, . ~ . |  . - log(usage) + ds + dp +dsp)                      #IV FE MODEL ESTIMATION, TWO PART FORMULA FOR IV

#NOW GATHER THE ESTIMATES FROM ALL 
rbind(ols = coef(summary(ols))[1,],
      fe = coef(summary(fe))[1, ],
      w2sls = coef(summary(ivfe))[1, ])

#TAKE NOTE! ENDOGENEITY IS VERY IMPORTANT. FOR THE FIRST MODEL THE COEFFICIENT 
#IS SIGNIFICANT AND POSITIVE. IT IS SLIGHTLY NEGATIVE FOR FE, BUT IN THAT MODEL 
#ENDOGENEITY STILL PERSISTS. IT ALSO MEANS THAT USAGE IS STRONGLY CORRELATED 
#WITH INDIVIDUAL EFFECTS. FINALLY, IT RISES FURTHER (IN ABS TERMS) ONCE W2SLS IS
#USED SO USAGE IS ALSO CORRELATED WITH ERROR TERM.

#NOW LETS TAKE A LOOK FOR ERROR COMPONENT MODELS. IT BECOMES CRUCIAL TO TACKLE 
#THE POTENTIAL PROBLEM OF CORRELATION BETWEEN SOME COVARIATES AND THE TWO ERROR
#COMPONENTS.

#REMINDER: GLS IS UNBIASED EFFICIENT ESTIMATOR IF NO COVARIATE IS CORRELATED 
#WITH THE COMPOSITE ERROR TERM. IF SOME COVARIATES ARE CORRELATED THAN NEITHER 
#GLS NOR WG IS APPROPRIATE. GLS IS NOT CONSISTENT, WITHIN IS CONSISTENT BUT NOT 
#EFFICIENT AND MAY NOT BE USED WHEN SOME VARIABLES DO NOT DISPLAY TEMPORAL
#VARIATION.

#THE BEST APPROACH WOULD BE TO USE IV BUT AT THE SAME TIME EXPLOIT TWO SOURCES 
#OF VARIABILITY IN THE PANEL. WE CAN USE IV ESTIMATOR (WG OR GLS) USING EXTERNAL
#INSTRUMENTS. HENCE, EC2SLS OR G2SLS CAN BE EMPLOYED. THEY VARY WITH RESPECT TO THE WAY
#INSTRUMENTAL VARIABLES ARE INTRODUCED INTO THE MODEL.

#WE WILL DIFERENTIATE BETWEEN: ENDOGENOUS, SIMPLE EXOGENOUS AND DOUBLY EXOGENOUS
#VARIABLES. THE LAST ONES ARE NEITHER CORRELATED WITH INDIVIDUAL EFFECTS NOR 
#WITH THE IDIOSYNCRATIC TERM.

##EC2SLS ESTIMATOR: ALL INSTRUMENTS ARE DOUBLY EXOGENOUS
##HT MODEL: NO ENDOGENOUS VARIABLES: ONLY SIMPLY OR DOUBLY EXOGENOUS. COVARIATES
#WITH TIME VARIATION ARE TRANSFORMED USED WITHIN FRAMEWORK, TIME-INVARIANT 
#INSTRUMENT ARE TAKEN AS THEY ARE. 
##G2SLS: THE SAME TRANSFORMATION IS APPLIED TO INSTRUMENTS, COVARATIES AND
#RESPONSE.

#WE USE A DATASET FOR 31 COUNTRIES FOR DATA FROM 1963 TO 1986 TO STUDY THE
#DETERMINANTS OF INTERNATIONAL TRADE FOR DEVELOPING COUNTRIES, IN PARTICULAR
#MEASURES OF PRICE AND INCOME ELASTICITIES.
data("ForeignTrade", package = "pder")                                          #LOAD DATA
?pder::ForeignTrade                                                             #DATASET DESCRIPTION

#WE CAN DIFFERENTIATE BETWEEN ENDOGENOUS AND EXOGENOUS REGRESSORS
w1 <- plm(imports~pmcpi + gnp + lag(imports) + lag(resimp)  |                   #FORMULA | FOR INSTRUMENTS
            lag(consump) + lag(cpi) + lag(income) + lag(gnp) + pm +
            lag(invest) + lag(money) + gnpw + pw + lag(reserves) +
            lag(exports) + trend + pgnp + lag(px),
          ForeignTrade, model = "within")
r1 <- update(w1, model = "random", random.method = "nerlove", 
             random.dfcor = c(1, 1), inst.method = "baltagi")
phtest(r1, w1)
#TAKE NOTE! WE CAN USE EITHER WITHIN OR EC2SLS ESTIMATOR AND TEST WITH HAUSMANN 
#EXOGENEITY OF INSTRUMENTS. THE HYPOTHESIS OF NO CORRELATION BETWEEN INSTRUMENTS
#AND INDIVIDUAL EFFECTS IMPLIES THAT WITHIN AND GLS ARE CONSISTENT  BUT GLS 
#IS MORE EFFICIENT. IF WE REJECT THE NULL, ONLY WITHIN IS CONSISTENT.
#WE REJECT THE NULL AT 5%. 

#WE CAN NOW USE TWO KINDS OF INSTRUMENTS:
#THOSE NOT CORRELATED WITH THE INDIVIDUAL EFFECTS AND CAN BE USED TWICE USING 
#THE WITHIN AND BETWEEN TRANSFORMATIONS
#TOSE CORRELATED WHICH CAN BE USED ONLY FOR WITHIN TRANSFORMATION

#NOW WE HAVE THREE-PART FORMULA.
r1b <- plm(imports ~ pmcpi + gnp + lag(imports) + lag(resimp) |
             lag(consump) + lag(cpi) + lag(income) + lag(px) + 
             lag(reserves) + lag(exports) | lag(gnp) + pm + 
             lag(invest) + lag(money) + gnpw + pw  + trend + pgnp,
           ForeignTrade, model = "random", inst.method = "baltagi", 
           random.method = "nerlove", random.dfcor = c(1, 1))
phtest(w1, r1b)
rbind(within = coef(w1), ec2sls = coef(r1b)[-1])
#TAKE NOTE! NOW WE DO NOT REJECT THE NULL. MOREOVER, COEFFICIENTS ARE SIMILAR.

#LETS CALCULATE THE ELASTICITY OF IMPORTS DEMAND GIVEN BY THE PRICE COEFFICIENT
#IN THE SHORT-TERM AND LONG-TERM.
elast <- sapply(list(w1, r1, r1b), 
                function(x) c(coef(x)["pmcpi"], 
                              coef(x)["pmcpi"] / (1 - coef(x)["lag(imports)"])))
dimnames(elast) <- list(c("ST", "LT"), c("w1", "r1", "r1b"))
elast

#WE CAN ALSO NOTE HOW THE STANDARD DEVIATION OF THE COEFFICIENTS HAS DECLINED.
rbind(within = coef(summary(w1))[, 2], 
      ec2sls = coef(summary(r1b))[-1, 2])

##HAUSMAN-TAYLOR ESTIMATOR
#WE NOW TURN OUR ATTENTION TO THE HAUSMAN-TAYLOR ESTIMATION. TO PROVIDE AN 
#EXAMPLE WE WILL USE THE DATASET ON INTERNATIONAL TRADE. IN GRAVITY MODELS WE 
#ASSUME THAT INTERNATIONAL TRADE BY VOLUME IS LINKED TO 'MASSES' OF BOTH 
#COUNTRIES AND DISTANCE BETWEEN THEM. NOTE THAT THE LATTER COULD BE A 
#TIME-INVARIANT REGRESSOR. SO WE NEED A PROPER ESTIMATOR. WE WILL USE YEARLY 
#OBSERVATIONS OF 91 PAIRS OF COUNTRIES FROM 1960 TO 2001.
rm(list=ls())
data("TradeEU", package = "pder")                                               #LOAD DATA
?pder::TradeEU                                                                  #DATASET DESCRIPTION

#WE FIRST ESTIMATE OLS, FE AND RE MODELS
ols <- plm(trade ~ gdp + dist + rer + rlf + sim + cee + emu + bor + lan, TradeEU, 
           model = "pooling", index = c("pair", "year"))
fe <- update(ols, model = "within")
re <- update(fe, model = "random")
phtest(re, fe)
#TAKE NOTE! WE DO NOT IDENTIFY SEVERAL PARAMETERS (FOR DIST, BOR AND LAN)
#RE MODEL SHOWS THAT A DISTANCE ELASTICITY OF BILATERAL TRADE OF ABOUT -0.6.
#HAVING COMMON BORDER OR LANGUAGE HAVE SIMILAR EFFECTS. HAUSMAN REJECTS RANDOM
#EFFECTS MODEL AT 5%.

#WE CAN ARGUE THAT AMONG TIME-INVARIANT REGRESSORS ONLY LAN IS CORRELATED WITH
#INDIVIDUAL EFFECTS. LET'S ESTIMATE TWO HT MODELS:
#1. RER IS THE ONLY DOUBLY EXOGENOUS VARIABLE (JUST-IDENTIFIED MODEL)
#2. GDP AND RLF ARE ALSO USED AS INSTRUMENTS (OVER-IDENTIFIED MODEL)
ht1 <- plm(trade ~ gdp + dist + rer + rlf + sim + cee + emu + bor + lan | 
             rer + dist + bor | gdp + rlf + sim + cee + emu + lan , 
           data = TradeEU, model = "random", index = c("pair", "year"), 
           inst.method = "baltagi", random.method = "ht")
ht2 <- update(ht1, trade ~ gdp + dist + rer + rlf + sim + cee + emu + bor + lan | 
                rer + gdp + rlf + dist + bor| sim + cee + emu + lan)
phtest(ht1, fe)
phtest(ht2, fe)
ht2am <- update(ht2, inst.method = "am")
phtest(ht2am, fe)
#CONSISTENCY OF EITHER SPECIFICATIONS IS NOT REJECTED BY THE HAUSMAN TEST.
#VALIDITY OF ADDITIONAL INSTRUMENTS IS NOT REJECTED BY HAUSMAN.

#FINALLY, LETS COMBINE ALL SPECIFICATIONS TOGHETHER.
texreg(list(ols, fe, re, ht1, ht2, ht2am),
       custom.model.names = c("OLS", "FE", "RE", "HT1", "HT2", "AM2"),
       caption = "Estimations of the gravity model.", label = "table:gravity",
       custom.gof.names = c("R$ ^ 2$", "Adj. R$ ^ 2$", "Num. obs.", "s\\_idios",
                            "s\\_id"),
       scriptsize = FALSE)
#USING HT ESTIMATORS WE ARE ABLE TO MEASURE THE INFLUENCE OF THE TIME-INVARIANT
#COVARIATES ON THE REGRESSAND.

##7. ESTIMATION AND TESTING OF DYNAMIC PANEL MODELS-----------------------------

##7.1 INTRODUCTION==============================================================
#DYNAMIC MODEL MEANS THAT THERE IS LAGGED DEPENDENT VARIABLE. THE USEFULNESS OF 
#SUCH MODELS IS SELF-EVIDENT. FOR INSTANCE, WE CAN ESTIMATE #NOMINAL AND REAL
#CONVERGENCE MODELS, LABOR FORCE ADJUSTMENTS, CONSUMPTION PATTERNS. APPROACHES 
#DISCUSSED BELOW ARE BEST SUITED TO MICRO PANELS, I.E. N>>T. STANDARD ESTIMATORS
#WON'T WORK DUE TO ENDOGENEITY, WE WILL NEED INSTRUMENTAL VARIABLES, MOSTLY GMM.

##7.2 RESEARCH QUESTION: QUANTIFY LINK BETWEEN WEALTH AND DEMOCRACY=============
##WE USE DATASET FROM ACEMOGLU, JOHNSON, ROBINSON AND YARED (2008) TO ILLUSTRATE
#THE DYNAMIC MODEL. WE HAVE DATA FOR 211 COUNTRIES OBSERVED EVERY 5 YEAR TIME 
#PERIODS SPANNING 1950-2000, T=11, N=211. ALTERNATIVELY, WE WILL USE DATA FOR 25
#COUNTRIES OBSERVED EVERY 25 YEARS (1850-2000), T=7, N=25.
rm(list=ls())
data("DemocracyIncome", package = "pder")                                       #LOAD FIRST SAMPLE
data("DemocracyIncome25", package = "pder")                                     #LOAD SECOND SAMPLE

#WE START BY INVESTIGATING CORRELATION.
set.seed(1)                                                                     #DECLARE SEED
di2000 <- subset(DemocracyIncome, year == "2000-2004",                          #ESTABLISH DATASET
                 select = c("democracy", "income", "country")) 
di2000 <- na.omit(di2000)                                                       #OMIT MISSING VALUES
di2000$country <- as.character(di2000$country)                                  #CONVERT TO CHAR
di2000$country[- c(2,5, 23, 16, 17, 22, 71,  125,                               #GET RID OF SOME LABELS
                   37, 43, 44, 79, 98, 105, 50, 120,  81, 129, 57, 58,99)] <- NA

#NOW WE DEVELOP SCATTER PLOT FOR INCOME AND DEMOCRACY.
ggplot(di2000, aes(income, democracy, label = country)) +                       #THIS ESTABLISHES PLOTS AND AXES 
  geom_point(size = 0.4) +                                                      #MANIPULATE DOTS ON PLOT
  geom_text(aes(y= democracy + sample(0.03 * c(-1, 1),                          #ADD TEST
                                      nrow(di2000), replace = TRUE)),
            size = 2) +
  theme(legend.text = element_text(size = 6),                                   #CHANGE THEME: LEGEND
        legend.title= element_text(size = 8),                                   #CHANGE THEME: TITLE
        axis.title = element_text(size = 8),                                    #CHANGE THEME: AXIS
        axis.text = element_text(size = 6))                                     #CHANGE THEME: AXIS TEXT

#NOW INSPECT THE PANEL ITSELF
pdim(DemocracyIncome)                                                           #INSPECT PANEL DIMENSION
head(DemocracyIncome, 4)                                                        #SHOW FIRST 4 LINES

#LETS START WITH ESTIMATING PANEL MODEL WITH TIME EFFECTS.
ols1 <- plm(democracy ~ lag(democracy) + lag(income) + year - 1,                #NO CONSTANT ONLY TIME EFFECTS 
           DemocracyIncome, index = c("country", "year"),                       #DATASET AND INDEXING
           model = "pooling", subset = sample == 1)                             #POOLING MODEL ON FIRST SAMPLE

#THIS IS AN EQUIVALENT FORM.
ols2  <- plm(democracy ~ lag(democracy) + lag(income),                          #MODEL FORMULA
           DemocracyIncome, index = c("country", "year"),                       #DATASET AND INDEXING
           model = "within", effect = "time",                                   #TIME EFFECTS
           subset = sample == 1)                                                #USE FIRST SAMPEL
summary(ols1)                                                                   #SUMMARIZE
coef(summary(ols2))
#TAKE NOTE! DUE TO NICKEL'S BIAS WE KNOW THAT THE ESTIMATE WILL SUFFER FROM
#POSITIVE BIAS. WE CANNOT USE FE TO ESTIMATE A DYNAMIC PANEL MODEL.

#WE TURN TO WITHIN GROUP ESTIMATOR, INTRODUCING TWO-WAY EFFECTS.
within <- update(ols, effect = "twoways")                                       #TWO-WAY ERROR COMPONENT
coef(summary(within))                                                           #COEFFICIENTS REPORT
#TAKE NOTE! NOW THE BIAS IS NEGATIVE, THE ESTIMATE FALLS CONSIDERABLY.

#HOW CAN WE DEAL WITH THIS SITUATION?
#1. USE ML METHODS (EXTREMELY SENSITIVE TO HYPOTHESES ON THE STARTING VALUES 
#OF EXPLANATORY VARIABLES).
#2. USE BC-LSDV IN THE SPIRIT OF KIVIET (1995).
#3. USE INSTRUMENTAL VARIABLES, LAGGED LEVELS OR DIFFERENCES.

#WE START WITH THE ANDERSON-HSIAO APPROACH. NOW BOTH REGRESSAND AND REGRESSOR 
#ARE DIFFERENCED. LAGGED ENDOGENOUS VARIABLE IN DIFFERENCES IS INSTRUMENTED WITH
#THE ENDOGENOUS IN LEVELS LAGGED TWO PERIODS.
ahsiao <- plm(diff(democracy) ~ lag(diff(democracy)) + lag(diff(income))        #MODEL ON DIFFERENCES
              + year - 1  |                                                     #TIME EFFECTS, NO CONST
                lag(democracy, 2) + lag(income, 2) + year - 1,                  #INSTRUMENTS IN LEVELS
              DemocracyIncome, index = c("country", "year"),                    #DATASET AND INDEXING
              model = "pooling", subset = sample == 1)                          #TYPE OF MODEL AND SUBSET
coef(summary(ahsiao))[1:2, ]                                                    #REPORT ESTIMATES
#TAKE NOTE! BEIGN CONSISTENT, THE COEFFICIENT SHOULD BE COMPRISED BETWEEN WG AND
#OLS, WHICH IS THE CASE. HOWEVER, THIS APPROACH IS INEFFICIENT. IT DOES NOT 
#ACCOUNT FOR CORRELATION INDUCED INTO ERRORS. IT ALSO DOES NOT EXPLORE FURTHER
#VALID INSTRUMENTS.

#NOW WE ESTIMATE DIFFERENCE GMM WITH ONESTEP AND TWOSTEP APPROACH
#TWOSTEP IS ROBUST TO HC AND SERIAL AUTOCORRELATION. THE VCV ESTIMATOR WILL
#STILL BE BIASED DUE AND FURTHER ROBUSTNESS CAN BE APPLIED.
diff1 <- pgmm(democracy ~ lag(democracy) + lag(income) |                        #MODEL EQUATION
                lag(democracy, 2:99)| lag(income, 2),                           #GMM INSTRUMENTS AND NORMAL INSTRUMENTS
              DemocracyIncome, index=c("country", "year"),                      #DATASET AND INDEXING
              model="onestep", effect="twoways", subset = sample == 1)          #TYPE OF MODEL, INDICATOR VARIABLES ADDED AS NORMAL INSTRUMENTS
coef(summary(diff1))                                                            #INITIAL ESTIMATES
diff2 <- update(diff1, model = "twosteps")                                      #TWO-STEP COVARIANCE MATRIX
coef(summary(diff2))                                                            #FINAL ESTIMATES
#TAKE NOTE! WE OBTAIN SIMILAR RESULTS AS IN AH. HOWEVER, FOR GMM THE NUMBER OF
#INSTRUMENTS GROW WITH THE DIMENSION T OF THE SAMPLE, QUADRATICALLY WITH T: 
#J=0.5(T-1)(T-2). THE WEIGHTING MATRICES ARE OF DIMENSION JxJ, HAVING J(J+1)/2 
#UNIQUE ELEMENTS. THUS PRECISION IS NOT GUARANTEED UNLESS N IS BIG WITH RESPECT 
#TO J. TO ILLUSTRATE THIS, CONSIDER 2SLS. THE MORE INSTRUMENTS YOU REGRESS IN
#THE FIRST STAGE, THE BETTER THE FIT, BUT THE MORE YOU CONVERGE TOWARDS OLS.
#THIS ILLUSTRATES PROBLEM WITH OVERFITTING.

pdim(DemocracyIncome25)                                                         #LOOK AT THIS DATASET

#ESTIMATE THE FULL DIFFERENCE GMM MODEL.
diff25 <- pgmm(democracy ~ lag(democracy) + lag(income) |                       #MODEL EQUATION
                 lag(democracy, 2:99) + lag(income, 2:99),                      #GMM INSTRUMENTS
               DemocracyIncome25, model = "twosteps")                           #DATASET AND VCV TYPE
#TAKE NOTE! HERE J=35, WHEN N=25.

#NOW CURTAIL INSTRUMENTS.
diff25lim <- pgmm(democracy ~ lag(democracy) + lag(income) |                    #MODEL EQUATION
                    lag(democracy, 2:4)+ lag(income, 2:4),                      #CURTAILED GMM INSTRUMENTS
                  DemocracyIncome, index=c("country", "year"),                  #DATASET AND INDEXING
                  model="twosteps", effect="twoways", subset = sample == 1)     #VCV TYPE, ECM TYPE AND SUBSETTING

#OR COLLAPSE THE INSTRUMENTS MATRIX.
diff25coll <- pgmm(democracy ~ lag(democracy) + lag(income) |                   #MODEL EQUATION
                     lag(democracy, 2:99)+ lag(income, 2:99),                   #GMM INSTRUMENTS
                   DemocracyIncome, index=c("country", "year"),                 #DATASET AND INDEXING
                   model="twosteps", effect="twoways", subset = sample == 1,    #VCV TYPE, ECM TYPE, SUBSETTING
                   collapse = TRUE)                                             #USE COLLAPSED MATRIX OF MOMENT CONDITIONS

sapply(list(diff25, diff25lim, diff25coll), function(x) coef(x)[1:2])           #GATHER RESULTS
#TAKE NOTE! ESTIMATES SHOW THAT THESE ASPECTS DO NOT INFLUENCE OUR ESTIMATES.

#IF INSTRUMENTS ARE WEAK, USE SYSTEM GMM. WE ADD ADDITIONAL MOMENT CONDITIONS 
#FROM THE LEVELS ESTIMATOR TO THE DIFFERENCE MODEL.
sys2 <- pgmm(democracy ~ lag(democracy) + lag(income) |                         #MODEL EQUATION 
               lag(democracy, 2:99)| lag(income, 2),                            #GMM AND NORMAL INSTRUMENTS
             DemocracyIncome, index = c("country", "year"),                     #DATASET AND INDEXING
             model = "twosteps", effect = "twoways",                            #VCV TYPE, ECM TYPE
             transformation = "ld")                                             #ADD ADDITIONAL MOMENT CONDITIONS IN LEVELS
coef(summary(sys2))                                                             #COLLECT ESTIMATES

#TAKE NOTE! AUTOREGRESSIVE COEFFICIENT IS CLOSE TO PREVIOUS ESTIMATES, BU
#INCOME COEFFICIENT IS NOW POSITIVE AND SIGNIFICANT

##FURTHER INFERENCE IS ALSO IMPORTANT. FOR GMM EVEN IF THE ESTIMATOR IS 
#CONSISTENT THE SAME MIGHT NOT BE TRUE FOR THE VARIANCE-COVARIANCE MATRIX.
#ESTIMATION MAY BE CONSISTENT ONLY UNDER CERTAIN HYPOTHESES: NO ERROR 
#CORRELATION AND MOMENTS VALIDITY. FOR VARIANCE-COVARIANCE THE STATE-OF-THE-ART 
#APPROACH IS TO USE WINDMEIJER (2005) CORRECTION.
sqrt(diag(vcov(diff2)))[1:2]                                                    #CLASSICAL ERRORS BIASED DOWNWARDS
sqrt(diag(vcovHC(diff2)))[1:2]                                                  #ROBUST ERRORS (CLEARLY SUPERIOR)

#WE MUST ALSO CHECK OVERIDENTIFICATION TESTS FOR VALID MOMENT CONDITIONS.
#PROPOSED BY SARGAN (1958) AND APPLIED BY HANSEN (1982) FOR GMM
sargan(diff2)                                                                   #J=55 AND K=11, J-K=44
sargan(sys2)                                                                    #J=66 AND K=12, J=K=54

#TAKE NOTE! THIS TEST WILL BE PARTICULARLY SENSITIVE FOR THE PROBLEM OF 
#INSTRUMENTS PROLIFERATION. P-VALUE WILL TEND TO BE VERY HIGH, 
#LEADING TO NON-REJECTING THE VALIDITY OF MOMENT CONDITIONS
sapply(list(diff25, diff25lim, diff25coll), function(x) sargan(x)[["p.value"]]) #COLLECT SARGAN FROM DIFFERENT DIFF MODELS

#GMM WILL BE NOT CONSISTENT UNLESS THE MOMENT CONDITIONS ARE VERIFIED. 
#INNOVATIONS MUST BE SERIALLY UNCORRELATED. TO CHECK THIS LET'S CARRY OUT THE
#TEST BY ARELLANO AND BOND (1991). WITH THE MODEL IN FIRST DIFFERENCES, 
#FIRST-ORDER CORRELATION TEST IS INAPPROPRIATE. THERE SHOULD BE CORRELATION BY 
#CONSTRUCTION. IN TURN, FOR SECOND AND HIGHER ORDERS THERE SHOULD NOT BE ANY.
mtest(diff2,order=1)
mtest(diff2,order=2)
mtest(diff2,order=3)
mtest(diff2,order=4)

#END OF FILE.