#########################################
# Econometrics SzD                      #
# Themme 4: Panel BMA                   #
#########################################

rm(list=ls())
source("libs.R")
source("utils.R")

# PART A. Introduction to Bayesian inference 

# 1. Conjugate prior
# Beta prior and binomial likelihood

ThetaGrid = seq(0.001,0.999,0.001)
# Settings
theta = 0.4
N     = 10
# Sample
y = rbinom(N,1,theta)
K = sum(y)

# prior
alpha0 <- 20
beta0  <- 50
mean0  <- alpha0/(alpha0+beta0)
sd0    <- sqrt((alpha0*beta0)/((alpha0+beta0)^2*(alpha0+beta0+1)))

prior <- dbeta(ThetaGrid,alpha0,beta0)

par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(ThetaGrid,prior,type="l",main=paste0("alpha=",alpha0,", beta=",beta0),xlab="",ylab="")

# likelihood
LL   <- dbinom(K,N,ThetaGrid)    

# posterior
alpha1    <- alpha0 + K
beta1     <- beta0 + (N-K)
mean1     <- alpha1/(alpha1+beta1)
sd1       <- sqrt((alpha1*beta1)/((alpha1+beta1)^2*(alpha1+beta1+1)))
posterior <- dbeta(ThetaGrid,alpha1,beta1) 

Check <- prior*LL/posterior
# why not 1? Compare to scaling constant
choose(N, K) * beta(alpha1,beta1)/beta(alpha0,beta0)

# A graph
temp <- data.frame(cbind(ThetaGrid,prior/mean(prior),LL/mean(LL),posterior/mean(posterior)))
colnames(temp) = c("ThetaGrid","prior","LL","posterior")
ggplot(data = temp, aes(x = ThetaGrid)) + 
  geom_line(aes(y = prior,colour="prior")) + 
  geom_line(aes(y = LL,colour="LL")) +
  geom_line(aes(y = posterior,colour="posterior")) +
  labs(title="Beta prior example", y="density function", x="Theta", caption="")+
  theme_minimal()+
  scale_colour_manual("", 
                      breaks = c("prior", "LL", "posterior"),
                      values = c("red", "green", "black")) 


# 2. Metropolis-Hastings algorithm
# Aim: find mean and sd of posterior distribution
###################################################

# function to calculate posterior given parameter value
posteriorValue <- function(theta){
  if(theta<0){theta<-0}
  if(theta>1){theta<-1}
  y <- dbeta(theta,alpha0,beta0)*dbinom(K,N,theta)
  return(y)
}

# settings
Ndraws         <- 10000
BurnIn         <- 1000
Nsim           <- Ndraws + BurnIn
ThetaDraws     <- rep(NA,Nsim)  
ThetaDraws[1]  <- 0.5                 # initial value
step           <- sd0                 # step length

# MH loop
for(i in 2:Nsim){
  theta.old = ThetaDraws[i-1]
  theta.new = theta.old + step*rnorm(1)
  posterior.old = posteriorValue(theta.old)
  posterior.new = posteriorValue(theta.new)
  if (posterior.new/posterior.old > runif(1)) {
    ThetaDraws[i] = theta.new
  } else {
    ThetaDraws[i] = theta.old
  }
} 

plot(ThetaDraws,type="l")
ThetaDraws = ThetaDraws[(BurnIn+1):Nsim]  # removing initial observations

mean2 = mean(ThetaDraws)
sd2   = sd(ThetaDraws)
c(mean1,mean2,sd1,sd2)

par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(density(ThetaDraws),main="Comparison of analytical and MH distribution",xlab="")
lines(ThetaGrid,posterior,col=2)




# PARTB: Bayesian model selection              
##################################################

# A. Loading data

# filename <- "dataJIE.csv"
# filename <- "http://web.sgh.waw.pl/~mrubas/AdvEcon/data/dataJIE.csv"
# data     <- read.csv(filename, header=TRUE, sep=";", dec=",", row.names="daty")
# names(data)

rm(list=ls())
filename <- "dataJIE.RData"
load(filename)
names(data)

# Legend		
# CA	current account (% GDP)	
# I	domestic interest rate (annual rate)	
# P	CPI QoQ (log growth rate) 	
# Y	GDP QoQ	
# YF	Foreign output QoQ	
# PF	Foreign prices QoQ	
# IF	Foreign interest rate (annual rate)	
# NER	Nominal Exchange Rate QoQ (increase stands for depreciation)	
# RER	Real Exchange Rate QoQ (increase stands for depreciation)	
#
# Countries:
# US  - United States
# EA  - euro area
# UK  - United Kingdom
# CAD - Canada
# AUS - Australia
#
# Source: Michele Ca' Zorzi, Marcin Kolasa and Michal Rubaszek: Exchange rate forecasting with DSGE models
# link: https://doi.org/10.1016/j.jinteco.2017.03.011

data[,11:50] <- 4*data[,11:50] # annualization
dates        <- as.Date(rownames(data),format="%d.%m.%Y")

data0 <- data[,c(5:10,15,20,25,30,35,45)] # only US variables and interest rates around the world

i <- 6 # variable number (interest rate in US)
y <- as.matrix(data0[,i]); colnames(y) <- names(data0)[i]
x <- as.matrix(cbind(data0[,-i])); colnames(x)=names(data0)[-i]
T <- dim(x)[1]
K <- dim(x)[2]
z <- cbind(y,x)


# B. Small BMA model with BMS Package

m1 = bms(z, mprior = "uniform", g = "UIP", user.int = F) #  g = "UIP" is the same as g=T (Unit information prior)
kable(coef(m1),digits=2)
summary(m1)
topmodels.bma(m1)[, 1:5]
image(m1)
plotModelsize(m1)

# controlling for the deviation of parameters from 0, g-Zelner hyperparameter
m2 = bms(z, mprior = "uniform", g = 1, user.int = F)
kable(cbind(coef(m1)[,1:2],coef(m2)[,1:2]),digits=2)

# controling for the model size (binomial prior)
m3 = bms(z, mprior = "fixed", mprior.size = 2, g = "UIP", user.int = F)
summary(m3)
kable(cbind(coef(m1)[,1:2],coef(m3)[,1:2]),digits=2)
plotModelsize(m3)
image(m3)
plotComp(Uniform = m1, Fixed = m3)


m4 = bms(z, mprior = "random", mprior.size = 2, g = "UIP", user.int = F)
summary(m4)
kable(cbind(coef(m3)[,1:2],coef(m4)[,1:2]),digits=2)
plotModelsize(m4)
image(m4)
plotComp(Uniform = m1, Fixed = m3, Random = m4)

pip_target    <- rep(0.1,K)
pip_target[6] <- 0.90
m5 = bms(z, mprior = "pip", mprior.size = pip_target, g = "UIP", user.int = F)
summary(m5)
coef(m5)
plotComp(Uniform = m1, Fixed = m3, Random = m4, PIP=m5)


# C. Bigger BMA model with a larger set of countries
data0 <- data[,c(1:25,41:45)] # we remove RER/foreign vars due to collinearity problem

i <- 10 # variable number (interest rate in US)
y <- as.matrix(data0[,i]); colnames(y) <- names(data0)[i]
x <- as.matrix(cbind(data0[,-i])); colnames(x)=names(data0)[-i]
T <- dim(x)[1]
K <- dim(x)[2]
z <- cbind(y,x)

# see page 10 of the below document for the details of MCMC settings
# browseURL("https://cran.r-project.org/web/packages/BMS/vignettes/bms.pdf")


mbig1 = bms(z, mprior = "uniform", g = "UIP", user.int = F)
summary(mbig1)

mbig2 = bms(z, mprior = "uniform", burn = 50000, iter = 1e+05, g = "UIP", user.int = F)
summary(mbig2)
coef(mbig2)
image(mbig2)

# 3. What determines economic growth

data(datafls)
help(datafls)
m1 <- bms(datafls, burn = 50000, iter = 1e+05, g = "UIP", 
          mprior = "uniform", mcmc = "bd", user.int = FALSE)

# Birth-death sampler ("bd"): 
# One of the K potential covariates is randomly chosen; 
# if the chosen covariate is in the current model Mi then it is dropped out
# If the chosen covariate is not in the current model Mi it is added up



